﻿Imports System.Windows.Threading
Imports Petzold.MusicSynthesis

Partial Public Class MainPage
    Inherits PhoneApplicationPage

    Private Const SAMPLE_RATE As Integer = 22050

    Private Shared ReadOnly MIN_PITCH As New Pitch(Note.C, 4)

    Private Shared ReadOnly MAX_PITCH As New Pitch(Note.C, 6)

    Private Shared ReadOnly NUM_PITCHES As Integer = MAX_PITCH.MidiNumber - MIN_PITCH.MidiNumber + 1

    Private stringInfos(NUM_PITCHES - 1) As StringInfo

    Private xDelta As Double

    Private mixer As Mixer

    Private touchDictionary As New Dictionary(Of Integer, Point)

    Public Sub New()
        InitializeComponent()

        mixer = New Mixer

        For index = 0 To NUM_PITCHES - 1
            Dim pitch As New Pitch(MIN_PITCH.MidiNumber + index)
            stringInfos(index) = New StringInfo(pitch)

            Dim oscillator As New SawtoothOscillator(SAMPLE_RATE) With {.Frequency = pitch.Frequency}

            Dim envelope As New AttackDecayEnvelope(SAMPLE_RATE) With {.Input = oscillator, .AttackTime = TimeSpan.FromSeconds(0.01), .DecayTime = TimeSpan.FromSeconds(5)}

            stringInfos(index).Envelope = envelope

            Dim mixerInput As New IntegerMixerInput(envelope) With {.Space = -1 + 2.0 * index / (NUM_PITCHES - 1)}

            mixer.Inputs.Add(mixerInput)
        Next index

        ' Install event handlers
        AddHandler Loaded, AddressOf OnLoaded
        AddHandler Touch.FrameReported, AddressOf OnTouchFrameReported

        Dim tmr As DispatcherTimer = New DispatcherTimer With {.Interval = TimeSpan.FromSeconds(0.1)}
        AddHandler tmr.Tick, AddressOf OnTimerTick
        tmr.Start()
    End Sub

    Private Sub OnLoaded(ByVal sender As Object, ByVal args As EventArgs)
        Dim count As Integer = NUM_PITCHES - 1
        xDelta = CInt((ContentPanel.ActualWidth - 4) / count)
        Dim xStart As Double = CInt((ContentPanel.ActualWidth - count * xDelta) / 2) + 0.5
        Dim cBrush As Brush = New SolidColorBrush(Colors.Red) ' C
        Dim fBrush As Brush = New SolidColorBrush(Colors.Blue) ' F
        Dim nBrush As Brush = TryCast(Me.Resources("PhoneForegroundBrush"), Brush) ' other naturals
        Dim aBrush As Brush = New SolidColorBrush(Colors.Gray) ' accidentals
        Dim x As Double = xStart
        Dim height As Double = ContentPanel.ActualHeight

        For Each stringInfo As StringInfo In stringInfos
            Dim pitch As Pitch = stringInfo.Pitch
            Dim yTop As Double = If(pitch.IsNatural, height / 3, 0)
            Dim yEnd As Double = If(pitch.IsNatural, height, 2 * height / 3)

            stringInfo.TopPoint = New Point(x, yTop)
            stringInfo.EndPoint = New Point(x, yEnd)

            Dim pathFigure As PathFigure = New PathFigure With {.StartPoint = stringInfo.TopPoint, .IsClosed = False, .IsFilled = False}

            stringInfo.BezierSegment = New QuadraticBezierSegment With {.Point1 = New Point(x, (yTop + yEnd) / 2), .Point2 = stringInfo.EndPoint}

            pathFigure.Segments.Add(stringInfo.BezierSegment)
            Dim pathGeometry As New PathGeometry
            pathGeometry.Figures.Add(pathFigure)
            Dim path As Path = New Path With {.Data = pathGeometry, .Stroke = If(pitch.Note = Note.C, cBrush, (If(pitch.Note = Note.F, fBrush, (If(pitch.IsNatural, nBrush, aBrush)))))}

            ContentPanel.Children.Add(path)
            x += xDelta
        Next stringInfo
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        Dim dynamicPcmStreamSource As New DynamicPcmStreamSource(SAMPLE_RATE, 15)
        dynamicPcmStreamSource.SampleProvider = mixer
        mediaElement.SetSource(dynamicPcmStreamSource)

        MyBase.OnNavigatedTo(args)
    End Sub

    Private Sub OnTouchFrameReported(ByVal sender As Object, ByVal args As TouchFrameEventArgs)
        Dim touchPoints As TouchPointCollection = args.GetTouchPoints(ContentPanel)

        For Each touchPoint In touchPoints
            Dim pt As Point = touchPoint.Position
            Dim id As Integer = touchPoint.TouchDevice.Id

            Select Case touchPoint.Action
                Case TouchAction.Down
                    For Each stringInfo As StringInfo In stringInfos
                        If pt.Y >= stringInfo.TopPoint.Y AndAlso pt.Y <= stringInfo.EndPoint.Y AndAlso Math.Abs(pt.X - stringInfo.TopPoint.X) < xDelta / 2 Then
                            stringInfo.Envelope.Attack()
                            Exit For
                        End If
                    Next stringInfo
                    touchDictionary.Add(id, pt)

                Case TouchAction.Move
                    Dim ptLast As Point = touchDictionary(id)

                    For Each stringInfo In stringInfos
                        Dim ptTop As Point = stringInfo.TopPoint
                        Dim ptEnd As Point = stringInfo.EndPoint

                        ' Corresponds with string on vertical axis
                        If (pt.Y >= ptTop.Y OrElse ptLast.Y >= ptTop.Y) AndAlso (pt.Y <= ptEnd.Y OrElse ptLast.Y <= ptEnd.Y) Then
                            If (pt.X > ptLast.X AndAlso pt.X >= ptTop.X AndAlso ptTop.X >= ptLast.X) OrElse (pt.X < ptLast.X AndAlso pt.X <= ptTop.X AndAlso ptTop.X <= ptLast.X) Then
                                stringInfo.Envelope.Attack()
                            End If
                        End If
                    Next stringInfo
                    touchDictionary(id) = pt

                Case TouchAction.Up
                    touchDictionary.Remove(id)
            End Select
        Next touchPoint
    End Sub

    Private Sub OnTimerTick(ByVal sender As Object, ByVal args As EventArgs)
        ' Wait until bezier segments are created
        If stringInfos(0).BezierSegment Is Nothing Then
            Exit Sub
        End If

        For Each stringInfo In stringInfos
            Dim x As Double = stringInfo.TopPoint.X
            Dim volume As Double = stringInfo.Envelope.Volume

            If volume = 0 Then
                If stringInfo.BezierSegment.Point1.X = x Then
                    Continue For
                Else
                End If
            Else
                If stringInfo.BezierSegment.Point1.X = x OrElse stringInfo.BezierSegment.Point1.X > x Then
                    x -= 10 * volume
                Else
                    x += 10 * volume
                End If
            End If

            stringInfo.BezierSegment.Point1 = New Point(x, stringInfo.BezierSegment.Point1.Y)
        Next stringInfo
    End Sub
End Class
